@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Time::HiRes qw(gettimeofday tv_interval);
 
-our $VERSION = '0.13'; # VERSION
+our $VERSION = '0.14'; # VERSION
 
 # exclude modules which we use ourselves
 my %excluded = map {$_=>1} (
@@ -46,6 +46,7 @@ our %opts = (
     force        => 0,
     hide_core    => 0,
     hide_noncore => 0,
+    show_memsize => 0,
 );
 
 # not yet
@@ -67,7 +68,17 @@ sub _inc_handler {
     #return (\*FH, );
 }
 
+sub _get_memsize {
+    # stolen from Memory::Usage
+    sysopen(my $fh, "/proc/$$/statm", 0) or die $!;
+    sysread($fh, my $line, 255) or die $!;
+    #my ($vsz, $rss, $share, $text, $crap, $data, $crap2) = split(/\s+/, $line,  7);
+    my ($vsz) = split(/\s+/, $line,  7);
+    $vsz;
+}
+
 my $start_time;
+my $memsize;
 my %inc_info;
 my $order;
 my $req_level = -1;
@@ -93,6 +104,11 @@ sub import {
         __PACKAGE__, " before others.\n",
     ) if @loaded > 5;
 
+    if ($opts{hide_core} || $opts{hide_noncore}) {
+        require Module::CoreList;
+        $excluded{$_} = 1 for keys %excluded_hide_core;
+    }
+
     #unshift @INC, \&_inc_handler;
     *CORE::GLOBAL::require = sub {
         my ($arg) = @_;
@@ -106,10 +122,12 @@ sub import {
             time   => 0,
         };
 
+        my $memsize1 = _get_memsize() if $opts{show_memsize};
         my $st = [gettimeofday];
         my $res;
         if (wantarray) { $res = [CORE::require $arg] } else { $res = CORE::require $arg }
         my $iv = tv_interval($st);
+        my $memsize2 = _get_memsize() if $opts{show_memsize};
 
         # still can't make exclusive time work
         #$req_times[$req_level] += $iv;
@@ -122,15 +140,14 @@ sub import {
         # inclusive time
         $inc_info{$arg}{time} = $iv;
 
+        # inclusive memory usage
+        $inc_info{$arg}{memsize} = $memsize2 - $memsize1
+            if $opts{show_memsize};
+
         $req_level--;
         if (wantarray) { return @$res } else { return $res }
     };
 
-    if ($opts{hide_core} || $opts{hide_noncore}) {
-        require Module::CoreList;
-        $excluded{$_} = 1 for keys %excluded_hide_core;
-    }
-
     $start_time = [gettimeofday];
 }
 
@@ -152,6 +169,7 @@ END {
     if ($begin_success || $opts{force}) {
 
         my $hc = $opts{hide_core} || $opts{hide_noncore};
+        my $sm = $opts{show_memsize};
 
         $stats .= "\n";
         $stats .= "# Start stats from Devel::EndStats:\n";
@@ -198,38 +216,48 @@ END {
         }
 
         if ($opts{verbose}) {
-            my $s = $opts{sort};
+            my $s = $opts{sort} // 'caller';
             my $sortsub;
             my $reverse;
             if ($s =~ /^(-?)l(?:ines)?/) {
                 $reverse = $1;
-                $sortsub = sub {($inc_info{$b}{$s}||0) <=> ($inc_info{$a}{$s}||0)};
+                $sortsub = sub {($inc_info{$a}{lines}||0) <=> ($inc_info{$b}{lines}||0)};
             } elsif ($s =~ /^(-)t(?:ime)?/) {
                 $reverse = $1;
-                $sortsub = sub {$inc_info{$b}{$s} <=> $inc_info{$a}{$s}};
+                $sortsub = sub {$inc_info{$b}{time} <=> $inc_info{$b}{time}};
             } elsif ($s =~ /^(-?)o(?:rder)?/) {
                 $reverse = $1;
-                $sortsub = sub {($inc_info{$a}{$s}||0) <=> ($inc_info{$b}{$s}||0)};
+                $sortsub = sub {($inc_info{$a}{seq}||0) <=> ($inc_info{$b}{seq}||0)};
             } elsif ($s =~ /^(-?)f(?:ile)?/) {
                 $reverse = $1;
                 $sortsub = sub {$a cmp $b};
-            } else {
+            } elsif ($s =~ /^(-?)(?:m|mem|memsize)/) {
+                $reverse = $1;
+                $sortsub = sub {($inc_info{$a}{memsize}||0) <=> ($inc_info{$b}{memsize}||0)};
+            } elsif ($s =~ /^(-?)(caller)/) {
                 # sort by caller;
                 $reverse = $s =~ /-/;
                 $sortsub = sub {$inc_info{$a}{$s} cmp $inc_info{$b}{$s}};
+            } else {
+                die "Unknown sort value";
             }
             my @rr = sort $sortsub keys %inc_info;
             @rr = reverse @rr if $reverse;
-            $stats .= "# Seq  Lines  Load Time        Module\n";
+            $stats .= "# Seq  Lines  Load Time       ".($sm ? "  MemSize":"")."  Module\n";
             for my $r (@rr) {
                 my $ii = $inc_info{$r};
                 next unless $ii->{lines};
                 next if $opts{hide_core} && defined($ii->{is_core}) && $ii->{is_core};
                 next if $opts{hide_noncore} && defined($ii->{is_core}) && !$ii->{is_core};
                 $ii->{time} ||= 0;
-                $stats .= sprintf "# %3s  %5d  %7.3fms(%3d%%)  %s (loaded by %s)\n",
-                     $ii->{order} || '?', $ii->{lines}, $ii->{time}*1000, $secs ? $ii->{time}/$secs*100 : 0,
-                         $r, ($ii->{caller} || "?");
+                $ii->{memsize} ||= 0;
+                $stats .= sprintf(
+                    "# %3s  %5d  %8.3fms(%3d%%)  ".($sm ? "%6.0fK" : "%s")."  %s (loaded by %s)\n",
+                    $ii->{order} || '?', $ii->{lines}, $ii->{time}*1000, $secs ? $ii->{time}/$secs*100 : 0,
+                    ($sm ? $ii->{memsize} : ""),
+                    $r,
+                    ($ii->{caller} || "?"),
+                );
             }
         }
 
@@ -258,7 +286,7 @@ Devel::EndStats - Display run time and dependencies after running code
 
 =head1 VERSION
 
-This document describes version 0.13 of Devel::EndStats (from Perl distribution Devel-EndStats), released on 2014-05-09.
+This document describes version 0.14 of Devel::EndStats (from Perl distribution Devel-EndStats), released on 2014-05-18.
 
 =head1 SYNOPSIS
 
@@ -349,6 +377,13 @@ Whether to hide core modules while listing modules in C<verbose> mode.
 
 Whether to hide non-core modules while listing modules in C<verbose> mode.
 
+=item * show_memsize => BOOL (default: 0)
+
+Whether to show memory usage information. Currently this is done by probing
+C</proc/$$/statm> because some other memory querying modules are unusable (e.g.
+L<Devel::SizeMe> currently segfaults on my system, C<Devel::InterpreterSize> is
+too heavy).
+
 =back
 
 =head1 FAQ
@@ -370,12 +405,12 @@ There are many modules on CPAN that can be used to generate dependency
 information for your code. Neil Bowers has written a
 L<review|http://neilb.org/reviews/dependencies.html> that covers most of them.
 
+=head1 KNOWN ISSUES
+
+* Timing and memory usage is inclusive instead of exclusive.
+
 =head1 TODO
 
-* Exclusive instead of inclusive timing for each require.
-
-* Stat: memory usage.
-
 * Stat: system/user time.
 
 * Stat: number of open files (sockets).
